-- #include "common.lua"

SPACAProjectileHandler = {
	shellNum = 1,
	shells = {},
	defaultShell = {active = false}
}
muzzleflash = LoadSprite("gfx/glare.png")
laserSprite = LoadSprite("gfx/laser.png")

function rndFloat(mi, ma)
	return mi + (ma-mi)*(math.random(0, 1000000)/1000000.0)
end

function VecDist(a,b)
	return VecLength(VecSub(a,b))
end

function init()
	RegisterTool("spaca", "SPACA", "MOD/vox/spaca.vox",3)
	SetBool("game.tool.spaca.enabled", true)
	SetFloat("game.tool.spaca.ammo", 12000)

	damage = 1.1
	gravity = Vec(0, -9.81, 0)
	velocity = 777
	reloadTime = 2.4
	shotDelay = 60/1000
	heavyShotDelay = 0.8
	
	ammo = 400
	unlimitedammo = GetBool("savegame.mod.unlimitedammo")
	moreAmmo = GetBool("savegame.mod.moreammo")
	if moreAmmo then
		ammo = 600
	end

	lefthand = false --todo: lefthand support
	zoomsense = 15000
	zoomlevel = 5
	maxzoom = 7
	minzoom = 0.5
	iszoomed = false
	scopeEnableKey = "X" --if you want to grab smth
	scopeEnabled = true
	nearWall = false
	
	
	shotmode = 1
	shotmodes = {"Default", "Auto", "Heavy", "Beam"} --default, auto heavy shot, beam in that order
	shotmodekey = "C"
	ammoModes = {4,1,8,1}
	--auto shot consumes 1, standard 4, heavy 8, beam 1 per frame

	for i=1, 40 do
		SPACAProjectileHandler.shells[i] = deepcopy(SPACAProjectileHandler.defaultShell)
	end

	shootTimer = 0
	reloadTimer = 0
	recoilTimer = 0
	recoilDelay = 0.16 --separated from shotDelay
	lightTimer = 0
	spreadTimer = 0 
	maxSpreadAngle = 1.67
	spreadDegree = 0
	spreadIncrease = 0.1
	
	--camera recoil operator
	qyr = 1.3
	qyrmin = qyr --fsrm will affect original value
	fsrm = 4 
	qxr = 1
	qxrmin = qxr
	qxrmax = 8
	qxrIncrease = 0.25 --increase for horizontal recoil
	qxrDecrease = 2.3
	vxr = 0 --amount of horizontal visual recoil (of the viewmodel)
	vxrdir = 0--visual recoil direction
	vpunch = 0
	vpunchfactor = 1 --the multiplier for stronger shots
	
	--recoil animation var
	animT = Transform() --for animating gun
	kickback = 0
	kickbackTarget = 6
	kickbackMax = 0.4
	kickup = 0
	kickupTarget = 10
	kickupDownCompensate = math.asin(math.rad(kickup))
	kicktilt = 0
	kickSideShake = 0
	sideShakeMult = 0
	
	--"spread"
	driftH = 0
	driftV = 0
	driftMult = 1
	
	vpunch = 0 --the fov "punch"
	vpunchTarget = 0 --for smoothing
	vpunchAmt = 8

	gunsound = LoadSound("MOD/snd/suppress_shot.ogg")
	cocksound = LoadSound("MOD/snd/awp_cock.ogg")
	dryfiresound = LoadSound("MOD/snd/dryfire.ogg")
	zoomsound = LoadSound("MOD/snd/awp_zoom.ogg")
	beepsound = LoadSound("MOD/snd/beep.ogg")
	beamsound = LoadLoop("MOD/snd/continuous_laser.ogg")
end

function inputMoving() --trying to move?
	if InputDown("left") or InputDown("right") or InputDown("up") or InputDown("down") or InputPressed("jump") then
		return true
	end
	return false
end

function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else
        copy = orig
    end
    return copy
end

function clamp(value, mi, ma)
	if value < mi then value = mi end
	if value > ma then value = ma end
	return value
end

function orientation(transform, sign)
	local fwd = VecNormalize(VecSub(TransformToParentPoint(transform, Vec(0, 0, -5)), transform.pos))
	local dir = Vec(0, 1*sign, 0)
	local orientationFactor = clamp(VecDot(dir, fwd) * 0.7 + 0.3, 0.0, 1.0)
	return orientationFactor
end
function canShoot() --no shooting in the car or computer screen
	if GetPlayerScreen() == 0 and GetPlayerVehicle() == 0 then
		return true
	end
	return false
end

function canUseTool() --because scoping disables the getbool
	if GetPlayerScreen() == 0 and GetPlayerVehicle() == 0 and GetPlayerGrabShape() == 0 then
		return true
	end
	return false
end

function GetAimPos(long)
	local ct = GetCameraTransform()
	local basedist = 150
	if long then basedist = 600 end
	local forwardPos = TransformToParentPoint(ct, Vec(0, 0, -basedist))
    local direction = VecSub(forwardPos, ct.pos)
    local distance = VecLength(direction)
	local direction = VecNormalize(direction)
	local hit, hitDistance = QueryRaycast(ct.pos, direction, distance)
	if hit then
		forwardPos = TransformToParentPoint(ct, Vec(0, 0, -hitDistance))
		distance = hitDistance
	end
	return forwardPos, hit, distance
end
function muzzleFlash() --set up the muzzleflash particle
	ParticleReset()
	ParticleType("plain")
	ParticleTile(0)
	ParticleColor(0.8,0.65,0.5)
	ParticleRadius(0.1,0.05)
	ParticleAlpha(1,1,"constant")
	ParticleEmissive(2,0.3)
	ParticleStretch(1)
	ParticleSticky(0)
	ParticleCollide(0)
end

function Shoot()
	if shootTimer > 0 or ammo == 0 then
		if ammo - ammoModes[shotmode] < 0 then
			PlaySound(dryfiresound, GetPlayerTransform().pos, 1, false)
			return
		end
		return
	end

	local ct = GetCameraTransform()
	aimpos, hit, distance = GetAimPos()

	local direction = VecSub(aimpos, toolPos)
	local direction2 = VecSub(aimpos, GetCameraTransform().pos)
	-- if nearWall then direction = direction2 end
	local muzzletrans = Transform(toolPos,QuatCopy(toolTrans.rot))
	local origin = toolPos
	-- if nearWall then origin = ct.pos end

	SPACAProjectileHandler.shells[SPACAProjectileHandler.shellNum] = deepcopy(SPACAProjectileHandler.defaultShell)
	loadedShell = SPACAProjectileHandler.shells[SPACAProjectileHandler.shellNum] 
	loadedShell.active = true
	
	--spread
	dirtrans = Transform(origin,QuatLookAt(Vec(), direction))
	-- if nearWall then dirtrans.rot = ct.rot end
	
	local roll = math.random()*360
	local spreadAngle = QuatEuler(math.random()*spreadDegree,0,roll)
	dirtrans.rot = QuatRotateQuat(dirtrans.rot, QuatEuler(driftV,-driftH,0)) --recoil based drift
	dirtrans.rot = QuatRotateQuat(dirtrans.rot,spreadAngle)
	direction = TransformToParentVec(dirtrans,Vec(0,0,-VecLength(direction)))

	loadedShell.pos = origin
	loadedShell.origPos = origin
	loadedShell.predictedBulletVelocity = VecScale(direction, (velocity/distance))
	loadedShell.travelDist = 0
	loadedShell.damage = damage
	loadedShell.killdist = 900 --don't indefinitely lag the map
	loadedShell.rendering = true
	loadedShell.renderdist = 3
	--damage modifier
	if shotmode ~= 1 then
		loadedShell.damage = damage/1.5
	end
	
	loadedShell.mode = shotmode --1 for default, 2 for auto, 3 for heavy, 4 for beam


	SPACAProjectileHandler.shellNum = (SPACAProjectileHandler.shellNum%#SPACAProjectileHandler.shells) +1

	--SpawnParticle(toolPos,TransformToParentVec(muzzletrans,Vec(0,0,-20)),0.1)
	--SpawnParticle("fire", toolPos, Vec(0, 1.0+math.random(1,10)*0.1, 0), 0.4, 0.15)
	--SpawnParticle("darksmoke", toolPos, Vec(0, 1.0+math.random(1,10)*0.1, 0), 0.25, 0.8)
	DrawSprite(muzzleflash,{pos=toolPos,rot=ct.rot},10,10,1,0.75,0.5,1,false,true)
	if shotmode == 3 then  PlaySound(gunsound, toolPos, 6, false)
	else
	PlaySound(gunsound, toolPos, 6, true) --bang
	end

	if not unlimitedammo then
		ammo = ammo - ammoModes[shotmode]
	end
	
	recoilTimer = recoilDelay
	if shotmode == 3 then recoilTimer = recoilDelay*2; driftMult = 2 end
	--recoil animation
	kickback = kickbackTarget
	kicktilt = math.random(-2,7)
	sideShakeMult = 1
	vpunchTarget = vpunchAmt
	if shotmode == 2 then 
		vpunchTarget = vpunchAmt/2 
		kickup = math.min(kickup+rndFloat(0,kickupTarget/2),kickupTarget)
		kickback = 0.4
	elseif shotmode == 3 then --heavy shot
		kickup = kickup + kickupTarget*2
		kickback = kickbackTarget*2
		sideShakeMult = 1.5
		vpunchTarget = vpunchAmt*2
	else
		kickup = kickup + kickupTarget
	end
	
	if not iszoomed then
		driftH = driftH + rndFloat(-1,1.2)*driftMult
		driftV = driftV + rndFloat(-1.4,1.6)*driftMult
	end
	
	shootTimer = shotDelay
	if shotmode == 3 then shootTimer = heavyShotDelay end
	
	spreadDegree = math.min(1.3,spreadDegree + spreadIncrease)
	local lightscale = 1.6
	if mode == 3 then lightscale = 4 end
	PointLight(toolPos, 1, 0.5, 0.1, lightscale) --muzzle flashes are instant right?
	
end

function laserBeam()
	local ct = GetCameraTransform()
	aimpos, hit, distance = GetAimPos()

	local direction = VecSub(aimpos, toolPos)
	local muzzletrans = Transform(toolPos,QuatCopy(toolTrans.rot))
	local origin = toolPos
	local lasertrans = Transform(origin,QuatLookAt(origin,aimpos))
	lasertrans.rot = QuatRotateQuat(lasertrans.rot,QuatEuler(driftV,-driftH,0))
	local laserdir = TransformToParentVec(lasertrans,Vec(0,0,-1))

	kickback = kickback + GetTimeStep()*4
	kickup = kickup + rndFloat(0,75*GetTimeStep())
	sideShakeMult = math.max(0.5,sideShakeMult)
	vpunch = math.min(vpunchAmt,vpunch + rndFloat(vpunchAmt/6,vpunchAmt/3))
	
	QueryRequire("physical")
	laserpos = TransformToParentPoint(lasertrans,Vec(0,0,-600))
	local laserhit, laserdist = QueryRaycast(origin,laserdir,600)
	if laserhit then
		laserpos = TransformToParentPoint(lasertrans,Vec(0,0,-laserdist))
	end


	PointLight(origin,1,0.5,0.1,rndFloat(1,4))
	-- DrawLine(origin,laserpos,1,0.8,0.4)
	MakeHole(laserpos,3,2.75,0.3)
	if math.random() < 0.1 then
		SpawnFire(laserpos)
	end
	PointLight(laserpos,1,0.5,0.1,rndFloat(1,10))
	PlayLoop(beamsound,origin,6)

	QueryRequire("physical") --dynamic optional
	local mi = VecAdd(laserpos,Vec(-4,-4,-4))
	local ma = VecAdd(laserpos,Vec(4,4,4))
	local bodies = QueryAabbBodies(mi, ma)	
	--loop through bodies
	for i,body in ipairs(bodies) do
		--compute body center and dist
		local bmi,bma = GetBodyBounds(body)
		local bc = VecLerp(bmi,bma,0.5) --get center
		local dir = VecSub(bc, laserpos)
		local dist = VecLength(dir)
		dir = VecScale(dir,1/dist)
		local mass = GetBodyMass(body)--get mass
		
		--dir forward
		dir = VecLerp(dir,TransformToParentVec(lasertrans,Vec(0,0,-100)),0.8)
		dir = VecNormalize(dir)
		
		local massScale = 1 - math.min(mass/(170),1)
		local distScale = 1 - math.min(dist/(4),1)^(1/3)
		local force = math.min(8*mass,4500)*distScale
		local add = VecScale(dir,force)
			
		ApplyBodyImpulse(body,laserpos,add)
	end
	--impact particle
	ParticleReset()
	ParticleType("plain")
	ParticleColor(1,0.69,0.3,1,0.37,0.1)
	ParticleCollide(0,1,"easeout",0,0.2)
	ParticleEmissive(2,0.3)
	ParticleRadius(0.06,0.02)
	ParticleStretch (0.75,0.28)
	for i = 0, 7 do
		SpawnParticle(laserpos,Vec(rndFloat(-4,4),rndFloat(-1,4),rndFloat(-4,4)),rndFloat(0.2,1))
	end
	
	--muzzle energy
	ParticleTile(5)
	ParticleRadius(0.17,0.04)
	ParticleStretch(1)
	ParticleCollide(0,1,"easeout")
	SpawnParticle(origin,TransformToParentVec(lasertrans,Vec(rndFloat(-4,4),rndFloat(-1,4),-rndFloat(45,60))),rndFloat(0.01,0.1))
	
	--laser sprite
	local t = Transform(VecLerp(origin,laserpos,0.5))
	local xAx = VecNormalize(VecSub(laserpos, origin))
	local zAx = VecNormalize(VecSub(origin, GetCameraTransform().pos))
	t.rot = QuatAlignXZ(xAx, zAx)
	DrawSprite(laserSprite, t, VecDist(origin,laserpos), 0.1+math.random()*0.07, 8, 6, 4, 1, true, true)
	DrawSprite(laserSprite, t, VecDist(origin,laserpos), 0.5, 1.0, 0.6, 0.2, 1, true, true)
	
	if not unlimitedammo then
		ammo = ammo - 1
	end
	
	if inputMoving() and not InputDown("crouch") and not iszoomed then
		driftH = driftH + rndFloat(-.3,.35)
		driftV = driftV + rndFloat(-.3,.3)
	end
	
	return true
end

function projectileParticle()
	ParticleReset()
	ParticleType("plain")
	ParticleColor(1,0.69,0.3,1,0.37,0.1)
	ParticleCollide(0,1,"easeout",0,0.2)
	ParticleEmissive(2,0.3)
	ParticleRadius(0.06,0.01)
	ParticleStretch (0.75,0.28)
end

function ProjectileOperations(projectile)
	projectile.predictedBulletVelocity = VecAdd(projectile.predictedBulletVelocity, (VecScale(gravity, GetTimeStep())))
	local point2 = VecAdd(projectile.pos, VecScale(projectile.predictedBulletVelocity, GetTimeStep()))
	local dir = VecNormalize(VecSub(point2, projectile.pos))
	local axis = Transform(projectile.pos,QuatLookAt(projectile.pos,point2)) --axis of bullet direction
	
	projectileParticle()
	if projectile.mode == 3 then ParticleRadius(0.2,0.05) end
	
	QueryRequire("physical")
	local hit, dist,normal,hitshape = QueryRaycast(projectile.pos, dir, VecLength(VecSub(point2, projectile.pos)))
	
	if hit then
		hitPos = VecAdd(projectile.pos, VecScale(dir, dist))
		hitbody = GetShapeBody(hitshape)
		
		for i=0, VecLength(VecSub(hitPos,projectile.pos)), 0.02 do
			SpawnParticle(TransformToParentPoint(axis,Vec(0,0,-i)),Vec(),0.1+GetTimeStep()*i*0.02)
		end
		
		projectile.active = false
		projectile.travelDist = projectile.travelDist + dist
		--SpawnParticle("smoke", hitPos, Vec(0, 1.0+math.random(1,10)*0.1, 0), projectile.damage, 1)
		local dmg = projectile.damage
		local lightsize = dmg*8
		if projectile.mode == 3 then
			Explosion(hitPos,3.2)
			lightsize = 3000
		else
			MakeHole(hitPos, projectile.damage, projectile.damage*0.9, 0)
		end
		PointLight(hitPos,1,0.5,0.1,lightsize)
		
		--GRAZE
		axis.pos = hitPos
		--bodies in holing distance
		QueryRequire("physical") --dynamic optional
		local mi = VecAdd(hitPos,Vec(-dmg*1.5,-dmg*1.5,-dmg*1.5))
		local ma = VecAdd(hitPos,Vec(dmg*1.5,dmg*1.5,dmg*1.5))
		local bodies = QueryAabbBodies(mi, ma)	
		--loop through bodies
		for i,body in ipairs(bodies) do
			--compute body center and dist
			local bmi,bma = GetBodyBounds(body)
			local bc = VecLerp(bmi,bma,0.5) --get center
			local dir = VecSub(bc, hitPos)
			local dist = VecLength(dir)
			dir = VecScale(dir,1/dist)
			local mass = GetBodyMass(body)--get mass
			
			--dir forward
			dir = VecLerp(dir,projectile.predictedBulletVelocity,0.8)
			dir = VecNormalize(dir)
			
			local massScale = 1 - math.min(mass/(170),1)
			--local distScale = 1 - math.min(dist/(dmg*1.2),1)
			local distScale = 1 - math.min(dist/(dmg*1.2),1)^(1/3)
			local force = math.min(17*mass,13500)*distScale
			local add = VecScale(dir,force)
			
			ApplyBodyImpulse(body,hitPos,add)
		end
	else
		if projectile.rendering then
			for i=0, VecLength(VecSub(point2,projectile.pos)), 0.02 do
				SpawnParticle(TransformToParentPoint(axis,Vec(0,0,-i)),Vec(),0.085+GetTimeStep()*i*0.02)
			end
		end
		-- DrawLine(projectile.pos, point2,1,0.75,0.7)
		PointLight(point2,1,0.5,0.1,4)
	end
	
	projectile.travelDist = projectile.travelDist + VecLength(VecScale(projectile.predictedBulletVelocity, GetTimeStep()))
	if projectile.travelDist > projectile.killdist then projectile.active = false end
	if projectile.travelDist > projectile.renderdist and not projectile.rendering then projectile.rendering = true end
    projectile.pos = point2
end

function Zoom()
	iszoomed = not iszoomed
	PlaySound(zoomsound, GetPlayerTransform().pos, 0.5, false)
end


function tick(dt)
	if iszoomed then
		SetString("game.player.tool", "spaca")
	end

	if GetString("game.player.tool") == "spaca" and canShoot() then
		if InputPressed("esc") then iszoomed = false end
		
		if InputPressed(scopeEnableKey) then 
			scopeEnabled = not scopeEnabled 
			if iszoomed then Zoom() end
		end
		
		if InputPressed(shotmodekey) then
			PlaySound(beepsound,GetPlayerPos(),1, false)
			shotmode = shotmode%#shotmodes + 1
		end
		
		if InputPressed("rmb") and scopeEnabled then Zoom() end
		
		local b = GetToolBody()
		local s = GetBodyShapes(b)[1] --gun is a single piece vox
		if b ~= 0 then
			local heightOffset = InputDown("crouch") and 0.125 or 0
			local offset = Transform(Vec(0, heightOffset, 0))
			if inspect then offset.rot = QuatEuler(40,-40,65) 
				offset.pos = VecAdd(offset.pos, Vec(-0.45,-0.7,0))
			end
			local extrasway = 0.8*(qxr/(qxrmax-qxrmin))^(1/2)
			toolTrans = GetBodyTransform(b)
			local t = Transform()
			kickupDownCompensate = math.asin(math.rad(kickup))*1.5
			local sideshake = math.sin(math.rad(kickSideShake))*sideShakeMult*0.005
			local rollshake = math.sin(math.rad(kickSideShake))*sideShakeMult*3
			t.pos = Vec(sideshake, heightOffset-kickupDownCompensate, math.min(kickbackMax,kickback))
			t.rot = QuatEuler(kickup+driftV, 0-driftH, rollshake)
			t.rot = QuatRotateQuat(t.rot,QuatEuler(0,0,kicktilt))
			animT.pos = VecLerp(animT.pos,t.pos,math.min(31*dt,0.6))
			animT.rot = QuatSlerp(animT.rot,t.rot,math.min(18*dt,0.6))
			SetToolTransform(animT,0.2+extrasway)
			toolPos = TransformToParentPoint(toolTrans, Vec(0.275, -0.275, -2.1))
			local sTrans = GetShapeWorldTransform(s)
			--verlet?
			verletVel = VecScale(VecSub(toolPos,prevToolPos),0.99)
			prevToolPos = toolPos
			-- if shotmode == 4 then 
				toolPos = VecAdd(toolPos,verletVel) 
			-- end
		end
		
		firing = false

		-- if shotmode==4 then
			-- aimpos, hit, distance = GetAimPos()
			-- local verletToolPos = VecAdd(toolPos,verletVel)
			-- local lineTrans = Transform(verletToolPos,QuatLookAt(verletToolPos,aimpos))
			-- laserRefPt = TransformToParentPoint(lineTrans,Vec(0,0,-150))
			-- verletLaserRef = VecScale(VecSub(laserRefPt,prevLaserRefPt),0.99)
			-- prevLaserRefPt = laserRefPt
			-- laserRefPt = VecAdd(laserRefPt,verletLaserRef)
		-- end

		if ammo > 0 and canUseTool() then
			if shotmode == 2 and InputDown("usetool") then --auto
				firing = true
				qyr = qyrmin
				if InputPressed("usetool") then
					qyr = qyr*fsrm
				end
			elseif InputDown("usetool") and shotmode == 4 then --beam
				laserBeam()
			elseif InputPressed("usetool") and ammo-ammoModes[shotmode] >=0 and shotmode~=4 then
				firing = true
			end
		end

		if InputPressed("lmb") and canUseTool() then
			if ammo == 0 or ammo-ammoModes[shotmode] < 0 then
				PlaySound(dryfiresound, GetPlayerTransform().pos, 1, false)
			end
		end--]]

		if firing then
			Shoot()
		end

		if iszoomed and scopeEnabled then
			toolPos = GetPlayerCameraTransform().pos
			local mousewheel = InputValue("mousewheel")
			if mousewheel ~= 0 then
				zoomlevel = clamp(zoomlevel - mousewheel/2, minzoom, maxzoom)
			end

			local pvel = GetPlayerVelocity()
			local mx, my = InputValue("mousedx"), -InputValue("mousedy")
	
			local ct = GetCameraTransform()
			local pt = GetPlayerTransform()
			
			local upAngle = orientation(Transform(pt.pos, ct.rot), 1)
			local downAngle = orientation(Transform(pt.pos, ct.rot), -1)
			if ((upAngle > 0.98 and my > 0) or (downAngle > 0.98 and my < 0)) then
			  my = 0
			end
	
			pt.pos[2] = pt.pos[2] + 1.8
			local target = Vec(mx/(zoomsense/zoomlevel), my/(zoomsense/zoomlevel), -1)
			target = TransformToParentVec(ct, target)
			ct.rot = QuatLookAt(ct, target)
	
			SetCameraTransform(Transform(pt.pos, ct.rot), zoomlevel*10)
			SetPlayerTransform(Transform(GetPlayerTransform().pos, ct.rot))
	
			SetPlayerVelocity(pvel)
		end
	
		if shootTimer > 0 or ammo == 0 then
			shootTimer = shootTimer - dt
		end
	SetCameraFov(GetInt("options.gfx.fov")+vpunch)
	end
	--recoil var timers
	kickback = math.max(0,kickback - kickback*30*dt)
	kickup = math.max(0,kickup - (kickup^.5)*26*dt)
	sideShakeMult = math.max(0,sideShakeMult-2*dt)
	if kicktilt < 0 then
		kicktilt = math.max(0,kicktilt+kicktilt*dt)
	elseif kicktilt > 0 then
		kicktilt = math.min(0,kicktilt - kicktilt*dt)
	end
	kickSideShake = kickSideShake%360 + dt*1666	
	--timers and stuff to stay active if tool is unequipped	
	if vpunch < vpunchTarget then vpunch = math.min(vpunchTarget,vpunch + 150*dt) else vpunch = vpunchTarget end
	vpunchTarget = math.max(0,vpunchTarget - vpunchTarget^1.5*7*dt)
	
	spreadDegree = math.max(spreadDegree - 4*dt,0)
	driftH = clamp(driftH,-6,6)
	if driftH < 0 then 
		driftH = math.min(0,driftH+4*dt)
	else
		driftH = math.max(0,driftH-4*dt)
	end
	driftV = clamp(driftV,-6,6)
	if driftV < 0 then 
		driftV = math.min(0,driftV+4*dt)
	else
		driftV = math.max(0,driftV-4*dt)
	end
	if not inputMoving() or InputDown("crouch") then
		driftMult = 0.5
	else
		driftMult = 1
	end
	
	for key, shell in ipairs(SPACAProjectileHandler.shells) do
		if shell.active then
			ProjectileOperations(shell)
		end
	end
end

function draw()
	if GetString("game.player.tool") == "spaca" and canShoot() then
		if iszoomed and scopeEnabled then
			UiPush()
				UiTranslate(UiCenter(), UiMiddle())
				UiAlign("center middle")
				UiImage("MOD/img/sfscope.png")
			UiPop()
		end

		if not unlimitedammo then
			UiPush()
				UiTranslate(UiCenter(), UiHeight()-60)
				UiAlign("center middle")
				UiColor(1, 1, 1)
				UiFont("bold.ttf", 32)
				UiTextOutline(0,0,0,1,0.1)
				UiText(ammo)

			UiPop()
		end

		UiPush()
			UiTranslate(UiCenter()/5, UiHeight()*0.9)
			UiAlign("center middle")
			UiColor(1, 1, 1)
			UiFont("bold.ttf", 24)
			UiTextOutline(0,0,0,1,0.1)
			if scopeEnabled then
				UiText("["..scopeEnableKey.."] Scope enabled")
			else
				UiText("["..scopeEnableKey.."] Scope disabled")
			end
			UiTranslate(0, -33)
		UiPop()
		
		UiPush()
			UiTranslate(UiCenter()/5, UiHeight()*0.925)
			UiAlign("center middle")
			UiColor(1, 1, 1)
			UiFont("bold.ttf", 24)
			UiTextOutline(0,0,0,1,0.1)
			UiText("["..shotmodekey.."] Shot mode: "..shotmodes[shotmode])
			UiTranslate(0,33)
			if inputMoving() and shotmode == 2 or inputMoving() and shotmode == 4 then
				UiColor(1,0.3,0.25)
				if not iszoomed and not InputDown("crouch") then
					UiText ("Unstable firing position")
				end
			end
		UiPop()
		
		UiPush() --nearwall ui
			UiAlign("center middle")
			nearWall = false
			if not iszoomed then
				QueryRequire("large")
				local ct = GetPlayerCameraTransform()
				local blocked,neardist = QueryRaycast(ct.pos,TransformToParentVec(ct,Vec(0,0,-1)),4.5)
				-- if blocked and shotmode ~= 4 then
					-- nearWall = true
					-- QueryRequire("large")
					-- local t = Transform(toolPos,ct.rot)
					-- t.rot = QuatRotateQuat(t.rot,QuatEuler(driftV,-driftH,0))
					-- local dir = TransformToParentVec(t,Vec(0,0,-1))
					-- dir = QuatRotateVec(QuatEuler(driftV,-driftH,0),dir)
					-- local blocked,neardist = QueryRaycast(toolPos,dir,100)
					-- local x,y,dist = UiWorldToPixel(VecAdd(toolPos,VecScale(dir,neardist)))
					-- UiTranslate(x,y)
					-- UiImage("ui/hud/crosshair-gun.png")	
					-- DebugCross(VecAdd(toolPos,TransformToParentVec(toolTrans,Vec(0,0,-neardist))))
				-- else
				local aimpos = GetAimPos()
				local dir = VecNormalize(VecSub(aimpos,toolPos))
				local dirtrans = Transform(toolPos,QuatLookAt(Vec(),dir))
				dirtrans.rot = QuatRotateQuat(dirtrans.rot, QuatEuler(driftV,-driftH,0))
				dir = TransformToParentVec(dirtrans,Vec(0,0,-1))
				local hit,dist = QueryRaycast(toolPos,dir,100)
				if not hit then dist = 100 end
				local x,y,dist = UiWorldToPixel(VecAdd(toolPos,VecScale(dir,dist)))
				UiTranslate(x,y)
				if shotmode ~=4 then UiImage("ui/hud/crosshair-gun.png") end
				UiImage("ui/hud/target.png")
				-- end
			end
		UiPop()
	end
end